# -*- coding: utf-8 -*-
"""
/***************************************************************************
 PythonModuleManager
                                 A QGIS plugin
 Manage Python modules
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2023-11-29
        git sha              : $Format:%H$
        copyright            : (C) 2023 by BABA Yoshihiko
        email                : babayoshihiko@mac.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from qgis.PyQt.QtCore import QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import QAction

# Initialize Qt resources from file resources.py
from .resources import *
# Import the code for the dialog
from .python_module_manager_dialog import PythonModuleManagerDialog
import os.path


class PythonModuleManager:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'PythonModuleManager_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            QCoreApplication.installTranslator(self.translator)

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&Python Module Manager')

        # Check if plugin was started the first time in current QGIS session
        # Must be set in initGui() to survive plugin reloads
        self.first_start = None

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('PythonModuleManager', message)


    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            # Adds plugin icon to Plugins toolbar
            self.iface.addToolBarIcon(action)

        if add_to_menu:
            self.iface.addPluginToMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/python_module_manager/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'Python Module Manager'),
            callback=self.run,
            parent=self.iface.mainWindow())

        # will be set False in run()
        self.first_start = True


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&Python Module Manager'),
                action)
            self.iface.removeToolBarIcon(action)


    def run(self):
        """Run method that performs all the real work"""

        # Create the dialog with elements (after translation) and keep reference
        # Only create GUI ONCE in callback, so that it will only load when the plugin is started
        if self.first_start == True:
            self.first_start = False
            self.dlg = PythonModuleManagerDialog()
            self.set_label_init()
            self.show_installed_packages()
            self.dlg.myPushButton1.clicked.connect(self.show_all_modules_2)
            self.dlg.myPushButton2.clicked.connect(self.show_installed_packages)
            self.dlg.myPushButton3.clicked.connect(self.install_module)
            self.dlg.myPushButton4.clicked.connect(self.show_package_path_2)

        # show the dialog
        self.dlg.show()
        # Run the dialog event loop
        result = self.dlg.exec_()
        # See if OK was pressed
        if result:
            # Do something useful here - delete the line containing pass and
            # substitute with your code.
            pass

    def show_installed_packages(self):
        import pkg_resources
        installed_packages = [pkg.key for pkg in pkg_resources.working_set]
        installed_packages.sort()
        self.dlg.myListWidget.clear()
        for package in installed_packages:
            self.dlg.myListWidget.addItem(package + " " + pkg_resources.get_distribution(package).version)

    def show_installed_packages_old(self):
        from pip._internal.utils.misc import get_installed_distributions
        installed_distributions = get_installed_distributions()
        installed_packages = []
        for dist in installed_distributions:
            package = {
                'name': dist.key,
                'version': dist.version,
                'location': dist.location
            }
            installed_packages.append(package)
        installed_packages.sort(key=lambda x: x['name'])
        self.dlg.myListWidget.clear()
        for package in installed_packages:
            self.dlg.myListWidget.addItem(package['name'] + ' ' + package['version'] + ' ' + package['location'])

    def set_label_init(self):
        import sys
        self.print_log("<html><b>Python version</b></html>")
        self.print_log(sys.version)
        self.print_log("<html><b>sys.executable</b>:</html>")
        self.print_log(sys.executable)
        self.print_log("<html><b>sys.path</b>:</html>")
        paths = sys.path
        possible_python = ""
        for path in paths:
            self.dlg.myTextEditLog.append(path)

    def print_log(self, message):
        self.dlg.myTextEditLog.append(message)

    def install_module(self):
        import pip
        module_names = self.dlg.myTextEdit.toPlainText().split("\n")

        for module_name in module_names:
            try:
                pip.main(["install", module_name])
                self.print_log(f"Successfully installed {module_name}")
                self.show_installed_packages()
            except Exception as e:
                self.print_log(f"Failed to install {module_name}. Error: {e}")

    def upgrade_all_modules(self):
        import pip
        import pkg_resources

        try:
            # Upgrade pip itself first
            pip.main(['install', '--upgrade', 'pip'])

            # Upgrade all installed packages
            installed_packages = [pkg.key for pkg in pkg_resources.working_set]
        
            for package in installed_packages:
                try:
                    pip.main(['install', '--upgrade', package])
                    self.print_log(f"Updated {package}.")
                except Exception as e:
                    self.print_log(f"Error updating modules: {e}")
                    break
            self.print_log("All modules updated successfully.")

        except Exception as e:
            self.print_log(f"Error updating modules: {e}")

    def show_package_path(self):
        import importlib.util
        import os
        
        package = self.dlg.myListWidget.currentItem().text()
        package_name = package.split(" ")[0]
        
        try:
            # Find the module specification
            spec = importlib.util.find_spec(package_name)
            
            if spec is not None:
                # Get the file path of the module
                package_path = os.path.abspath(spec.origin)
                self.print_log(f"Pacakge '{package_name}' is at {package_path}.")
            else:
                self.print_log(f"Package '{package_name}' not found.")
                return None
        except ImportError as e:
            self.print_log(f"Error: {e}")
            return None

    def show_package_path_2(self):
        import pkg_resources
        package = self.dlg.myListWidget.currentItem().text()
        package_name = package.split(" ")[0]
        self.print_log(pkg_resources.get_distribution(package_name).location)
        
    def show_all_modules(self):
        import importlib
        import pkgutil
        
        package = self.dlg.myListWidget.currentItem().text()
        package_name = package.split(" ")[0]
        
        
        package = importlib.import_module(package_name)
        
        module_iterator = pkgutil.iter_modules(package.__path__)
        
        for _, module_name, _ in module_iterator:
            full_module_name = f"{package_name}.{module_name}"
            self.print_log(full_module_name)

    def show_all_modules_2(self):
        import pkg_resources
        package = self.dlg.myListWidget.currentItem().text()
        package_name = package.split(" ")[0]
        distribution = pkg_resources.get_distribution(package_name)
        
        for entry_point in distribution.get_entry_map().values():
            for entry_point in entry_point.values():
                if entry_point.module_name:
                    self.print_log(entry_point.module_name)
